/****************************************************************************
** Meta object code from reading C++ file 'accountsettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../opencloud-desktop/src/gui/accountsettings.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'accountsettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC15AccountSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::AccountSettings::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC15AccountSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::AccountSettings",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "C++ only",
        "showIssuesList",
        "",
        "unsyncedSpacesChanged",
        "syncedSpacesChanged",
        "connectionLabelChanged",
        "notificationsChanged",
        "slotAccountStateChanged",
        "slotSpacesUpdated",
        "slotAddFolder",
        "slotEnableCurrentFolder",
        "Folder*",
        "folder",
        "terminate",
        "slotForceSyncCurrentFolder",
        "slotRemoveCurrentFolder",
        "showSelectiveSyncDialog",
        "slotFolderWizardAccepted",
        "slotDeleteAccount",
        "slotToggleSignInState",
        "markNotificationsRead",
        "focusNext",
        "focusPrevious",
        "accountState",
        "AccountState*",
        "model",
        "QSortFilterProxyModel*",
        "unsyncedSpaces",
        "uint64_t",
        "syncedSpaces",
        "connectionLabel",
        "accountStateIconGlype",
        "notifications",
        "QSet<Notification>",
        "ModalWidgetSizePolicy",
        "Minimum",
        "Expanding"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'showIssuesList'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'unsyncedSpacesChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'syncedSpacesChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'connectionLabelChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'notificationsChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAccountStateChanged'
        QtMocHelpers::SlotData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSpacesUpdated'
        QtMocHelpers::SlotData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAddFolder'
        QtMocHelpers::SlotData<void()>(15, 8, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEnableCurrentFolder'
        QtMocHelpers::SlotData<void(Folder *, bool)>(16, 8, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Bool, 19 },
        }}),
        // Slot 'slotEnableCurrentFolder'
        QtMocHelpers::SlotData<void(Folder *)>(16, 8, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotForceSyncCurrentFolder'
        QtMocHelpers::SlotData<void(Folder *)>(20, 8, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotRemoveCurrentFolder'
        QtMocHelpers::SlotData<void(Folder *)>(21, 8, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'showSelectiveSyncDialog'
        QtMocHelpers::SlotData<void(Folder *)>(22, 8, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotFolderWizardAccepted'
        QtMocHelpers::SlotData<void()>(23, 8, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDeleteAccount'
        QtMocHelpers::SlotData<void()>(24, 8, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotToggleSignInState'
        QtMocHelpers::SlotData<void()>(25, 8, QMC::AccessProtected, QMetaType::Void),
        // Slot 'markNotificationsRead'
        QtMocHelpers::SlotData<void()>(26, 8, QMC::AccessProtected, QMetaType::Void),
        // Method 'focusNext'
        QtMocHelpers::MethodData<void()>(27, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'focusPrevious'
        QtMocHelpers::MethodData<void()>(28, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'accountState'
        QtMocHelpers::PropertyData<AccountState*>(29, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'model'
        QtMocHelpers::PropertyData<QSortFilterProxyModel*>(31, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'unsyncedSpaces'
        QtMocHelpers::PropertyData<uint64_t>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'syncedSpaces'
        QtMocHelpers::PropertyData<uint64_t>(35, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'connectionLabel'
        QtMocHelpers::PropertyData<QString>(36, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'accountStateIconGlype'
        QtMocHelpers::PropertyData<QChar>(37, QMetaType::QChar, QMC::DefaultPropertyFlags, 3),
        // property 'notifications'
        QtMocHelpers::PropertyData<QSet<Notification>>(38, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ModalWidgetSizePolicy'
        QtMocHelpers::EnumData<enum ModalWidgetSizePolicy>(40, 40, QMC::EnumIsScoped).add({
            {   41, ModalWidgetSizePolicy::Minimum },
            {   42, ModalWidgetSizePolicy::Expanding },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<AccountSettings, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OCC::AccountSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15AccountSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15AccountSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC15AccountSettingsE_t>.metaTypes,
    nullptr
} };

void OCC::AccountSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AccountSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->showIssuesList(); break;
        case 1: _t->unsyncedSpacesChanged(); break;
        case 2: _t->syncedSpacesChanged(); break;
        case 3: _t->connectionLabelChanged(); break;
        case 4: _t->notificationsChanged(); break;
        case 5: _t->slotAccountStateChanged(); break;
        case 6: _t->slotSpacesUpdated(); break;
        case 7: _t->slotAddFolder(); break;
        case 8: _t->slotEnableCurrentFolder((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->slotEnableCurrentFolder((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 10: _t->slotForceSyncCurrentFolder((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 11: _t->slotRemoveCurrentFolder((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 12: _t->showSelectiveSyncDialog((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 13: _t->slotFolderWizardAccepted(); break;
        case 14: _t->slotDeleteAccount(); break;
        case 15: _t->slotToggleSignInState(); break;
        case 16: _t->markNotificationsRead(); break;
        case 17: _t->focusNext(); break;
        case 18: _t->focusPrevious(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)()>(_a, &AccountSettings::showIssuesList, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)()>(_a, &AccountSettings::unsyncedSpacesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)()>(_a, &AccountSettings::syncedSpacesChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)()>(_a, &AccountSettings::connectionLabelChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AccountSettings::*)()>(_a, &AccountSettings::notificationsChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< AccountState* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSortFilterProxyModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<AccountState**>(_v) = _t->_accountState; break;
        case 1: *reinterpret_cast<QSortFilterProxyModel**>(_v) = _t->_sortModel; break;
        case 2: *reinterpret_cast<uint64_t*>(_v) = _t->unsyncedSpaces(); break;
        case 3: *reinterpret_cast<uint64_t*>(_v) = _t->syncedSpaces(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->connectionLabel(); break;
        case 5: *reinterpret_cast<QChar*>(_v) = _t->accountStateIconGlype(); break;
        case 6: *reinterpret_cast<QSet<Notification>*>(_v) = _t->notifications(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::AccountSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::AccountSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC15AccountSettingsE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int OCC::AccountSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void OCC::AccountSettings::showIssuesList()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::AccountSettings::unsyncedSpacesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::AccountSettings::syncedSpacesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void OCC::AccountSettings::connectionLabelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::AccountSettings::notificationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
