/****************************************************************************
** Meta object code from reading C++ file 'qmlcredentials.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../opencloud-desktop/src/gui/creds/qmlcredentials.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qmlcredentials.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC14QmlCredentialsE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::QmlCredentials::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC14QmlCredentialsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::QmlCredentials",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Abstract class",
        "readyChanged",
        "",
        "logOutRequested",
        "isRefreshChanged",
        "host",
        "QUrl",
        "displayName",
        "ready",
        "isRefresh"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'readyChanged'
        QtMocHelpers::SignalData<void() const>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'logOutRequested'
        QtMocHelpers::SignalData<void() const>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isRefreshChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'host'
        QtMocHelpers::PropertyData<QUrl>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'displayName'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant | QMC::Final),
        // property 'ready'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 0),
        // property 'isRefresh'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QmlCredentials, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OCC::QmlCredentials::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14QmlCredentialsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14QmlCredentialsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC14QmlCredentialsE_t>.metaTypes,
    nullptr
} };

void OCC::QmlCredentials::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QmlCredentials *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->readyChanged(); break;
        case 1: _t->logOutRequested(); break;
        case 2: _t->isRefreshChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QmlCredentials::*)() const>(_a, &QmlCredentials::readyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlCredentials::*)() const>(_a, &QmlCredentials::logOutRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlCredentials::*)()>(_a, &QmlCredentials::isRefreshChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->_host; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->_displayName; break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isReady(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isRefresh(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 3: _t->setIsRefresh(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::QmlCredentials::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::QmlCredentials::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC14QmlCredentialsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::QmlCredentials::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void OCC::QmlCredentials::readyChanged()const
{
    QMetaObject::activate(const_cast< OCC::QmlCredentials *>(this), &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::QmlCredentials::logOutRequested()const
{
    QMetaObject::activate(const_cast< OCC::QmlCredentials *>(this), &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void OCC::QmlCredentials::isRefreshChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN3OCC19QmlOAuthCredentialsE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::QmlOAuthCredentials::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC19QmlOAuthCredentialsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::QmlOAuthCredentials",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "C++ only",
        "requestRestart",
        "",
        "copyAuthenticationUrlToClipboard",
        "openAuthenticationUrlInBrowser",
        "readyChanged",
        "isValid"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestRestart'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'copyAuthenticationUrlToClipboard'
        QtMocHelpers::MethodData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'openAuthenticationUrlInBrowser'
        QtMocHelpers::MethodData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isValid'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 0x70000000 | 11),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QmlOAuthCredentials, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OCC::QmlOAuthCredentials::staticMetaObject = { {
    QMetaObject::SuperData::link<QmlCredentials::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19QmlOAuthCredentialsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19QmlOAuthCredentialsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC19QmlOAuthCredentialsE_t>.metaTypes,
    nullptr
} };

void OCC::QmlOAuthCredentials::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QmlOAuthCredentials *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestRestart(); break;
        case 1: _t->copyAuthenticationUrlToClipboard(); break;
        case 2: _t->openAuthenticationUrlInBrowser(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QmlOAuthCredentials::*)()>(_a, &QmlOAuthCredentials::requestRestart, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isValid(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::QmlOAuthCredentials::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::QmlOAuthCredentials::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC19QmlOAuthCredentialsE_t>.strings))
        return static_cast<void*>(this);
    return QmlCredentials::qt_metacast(_clname);
}

int OCC::QmlOAuthCredentials::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QmlCredentials::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void OCC::QmlOAuthCredentials::requestRestart()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace CheckNotifySignalValidity_ZN3OCC19QmlOAuthCredentialsE {
template<typename T> using has_nullary_readyChanged = decltype(std::declval<T>().readyChanged());
template<typename T> using has_unary_readyChanged = decltype(std::declval<T>().readyChanged(std::declval<bool>()));
static_assert(qxp::is_detected_v<has_nullary_readyChanged, OCC::QmlOAuthCredentials> || qxp::is_detected_v<has_unary_readyChanged, OCC::QmlOAuthCredentials>,
              "NOTIFY signal readyChanged does not exist in class (or is private in its parent)");
}
QT_WARNING_POP
