/********************************************************************************
** Form generated from reading UI file 'accountconfiguredwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACCOUNTCONFIGUREDWIZARDPAGE_H
#define UI_ACCOUNTCONFIGUREDWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_AccountConfiguredWizardPage
{
public:
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer;
    QLabel *youreAllSetLabel;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *advancedConfigGroupBox;
    QHBoxLayout *horizontalLayout;
    QWidget *advancedConfigGroupBoxContentWidget;
    QVBoxLayout *verticalLayout_4;
    QGroupBox *syncModeGroupBox;
    QVBoxLayout *syncModeGroupBoxLayout;
    QLabel *label;
    QRadioButton *syncEverythingRadioButton;
    QRadioButton *configureSyncManuallyRadioButton;
    QGroupBox *localDirectoryGroupBox;
    QVBoxLayout *verticalLayout_2;
    QLabel *chooseLocalDownloadDirectoryLabel;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *localDirectoryLineEdit;
    QToolButton *chooseLocalDirectoryButton;
    QToolButton *resetLocalDirectoryButton;

    void setupUi(QWidget *AccountConfiguredWizardPage)
    {
        if (AccountConfiguredWizardPage->objectName().isEmpty())
            AccountConfiguredWizardPage->setObjectName("AccountConfiguredWizardPage");
        AccountConfiguredWizardPage->resize(460, 432);
        verticalLayout = new QVBoxLayout(AccountConfiguredWizardPage);
        verticalLayout->setObjectName("verticalLayout");
        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        verticalLayout->addItem(verticalSpacer);

        youreAllSetLabel = new QLabel(AccountConfiguredWizardPage);
        youreAllSetLabel->setObjectName("youreAllSetLabel");
        youreAllSetLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(youreAllSetLabel);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        verticalLayout->addItem(verticalSpacer_2);

        advancedConfigGroupBox = new QGroupBox(AccountConfiguredWizardPage);
        advancedConfigGroupBox->setObjectName("advancedConfigGroupBox");
        advancedConfigGroupBox->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(advancedConfigGroupBox->sizePolicy().hasHeightForWidth());
        advancedConfigGroupBox->setSizePolicy(sizePolicy);
        advancedConfigGroupBox->setFlat(false);
        advancedConfigGroupBox->setCheckable(true);
        advancedConfigGroupBox->setChecked(false);
        horizontalLayout = new QHBoxLayout(advancedConfigGroupBox);
        horizontalLayout->setSpacing(60);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 24, 0, 0);
        advancedConfigGroupBoxContentWidget = new QWidget(advancedConfigGroupBox);
        advancedConfigGroupBoxContentWidget->setObjectName("advancedConfigGroupBoxContentWidget");
        verticalLayout_4 = new QVBoxLayout(advancedConfigGroupBoxContentWidget);
        verticalLayout_4->setSpacing(18);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(9, 9, 9, 9);
        syncModeGroupBox = new QGroupBox(advancedConfigGroupBoxContentWidget);
        syncModeGroupBox->setObjectName("syncModeGroupBox");
        syncModeGroupBox->setFlat(true);
        syncModeGroupBoxLayout = new QVBoxLayout(syncModeGroupBox);
        syncModeGroupBoxLayout->setSpacing(3);
        syncModeGroupBoxLayout->setObjectName("syncModeGroupBoxLayout");
        syncModeGroupBoxLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(syncModeGroupBox);
        label->setObjectName("label");

        syncModeGroupBoxLayout->addWidget(label);

        syncEverythingRadioButton = new QRadioButton(syncModeGroupBox);
        syncEverythingRadioButton->setObjectName("syncEverythingRadioButton");
        syncEverythingRadioButton->setChecked(true);

        syncModeGroupBoxLayout->addWidget(syncEverythingRadioButton);

        configureSyncManuallyRadioButton = new QRadioButton(syncModeGroupBox);
        configureSyncManuallyRadioButton->setObjectName("configureSyncManuallyRadioButton");

        syncModeGroupBoxLayout->addWidget(configureSyncManuallyRadioButton);


        verticalLayout_4->addWidget(syncModeGroupBox);

        localDirectoryGroupBox = new QGroupBox(advancedConfigGroupBoxContentWidget);
        localDirectoryGroupBox->setObjectName("localDirectoryGroupBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(localDirectoryGroupBox->sizePolicy().hasHeightForWidth());
        localDirectoryGroupBox->setSizePolicy(sizePolicy1);
        localDirectoryGroupBox->setStyleSheet(QString::fromUtf8("QGroupBox {border:0;}"));
        localDirectoryGroupBox->setTitle(QString::fromUtf8(""));
        localDirectoryGroupBox->setFlat(true);
        verticalLayout_2 = new QVBoxLayout(localDirectoryGroupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        chooseLocalDownloadDirectoryLabel = new QLabel(localDirectoryGroupBox);
        chooseLocalDownloadDirectoryLabel->setObjectName("chooseLocalDownloadDirectoryLabel");

        verticalLayout_2->addWidget(chooseLocalDownloadDirectoryLabel);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        localDirectoryLineEdit = new QLineEdit(localDirectoryGroupBox);
        localDirectoryLineEdit->setObjectName("localDirectoryLineEdit");

        horizontalLayout_3->addWidget(localDirectoryLineEdit);

        chooseLocalDirectoryButton = new QToolButton(localDirectoryGroupBox);
        chooseLocalDirectoryButton->setObjectName("chooseLocalDirectoryButton");
        chooseLocalDirectoryButton->setText(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/client/resources/core/more.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        chooseLocalDirectoryButton->setIcon(icon);

        horizontalLayout_3->addWidget(chooseLocalDirectoryButton);

        resetLocalDirectoryButton = new QToolButton(localDirectoryGroupBox);
        resetLocalDirectoryButton->setObjectName("resetLocalDirectoryButton");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/client/resources/core/undo.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        resetLocalDirectoryButton->setIcon(icon1);

        horizontalLayout_3->addWidget(resetLocalDirectoryButton);


        verticalLayout_2->addLayout(horizontalLayout_3);


        verticalLayout_4->addWidget(localDirectoryGroupBox);


        horizontalLayout->addWidget(advancedConfigGroupBoxContentWidget);


        verticalLayout->addWidget(advancedConfigGroupBox);


        retranslateUi(AccountConfiguredWizardPage);

        QMetaObject::connectSlotsByName(AccountConfiguredWizardPage);
    } // setupUi

    void retranslateUi(QWidget *AccountConfiguredWizardPage)
    {
        youreAllSetLabel->setText(QCoreApplication::translate("AccountConfiguredWizardPage", "\342\234\223 You're all set!", nullptr));
        advancedConfigGroupBox->setTitle(QCoreApplication::translate("AccountConfiguredWizardPage", "Advanced configuration", nullptr));
        syncModeGroupBox->setTitle(QString());
        label->setText(QCoreApplication::translate("AccountConfiguredWizardPage", "Configure files download:", nullptr));
        syncEverythingRadioButton->setText(QCoreApplication::translate("AccountConfiguredWizardPage", "Synchronize all existing spaces", nullptr));
#if QT_CONFIG(tooltip)
        configureSyncManuallyRadioButton->setToolTip(QCoreApplication::translate("AccountConfiguredWizardPage", "After completing this wizard, you can set up folder synchronization manually.", nullptr));
#endif // QT_CONFIG(tooltip)
        configureSyncManuallyRadioButton->setText(QCoreApplication::translate("AccountConfiguredWizardPage", "Configure synchronization manually", nullptr));
        chooseLocalDownloadDirectoryLabel->setText(QCoreApplication::translate("AccountConfiguredWizardPage", "Choose local download directory:", nullptr));
#if QT_CONFIG(tooltip)
        resetLocalDirectoryButton->setToolTip(QCoreApplication::translate("AccountConfiguredWizardPage", "Restore default value", nullptr));
#endif // QT_CONFIG(tooltip)
        resetLocalDirectoryButton->setText(QString());
        (void)AccountConfiguredWizardPage;
    } // retranslateUi

};

namespace Ui {
    class AccountConfiguredWizardPage: public Ui_AccountConfiguredWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ACCOUNTCONFIGUREDWIZARDPAGE_H
