/****************************************************************************
** Meta object code from reading C++ file 'bandwidthmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../opencloud-desktop/src/libsync/bandwidthmanager.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bandwidthmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC16BandwidthManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::BandwidthManager::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC16BandwidthManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::BandwidthManager",
        "registerUploadDevice",
        "",
        "UploadDevice*",
        "unregisterUploadDevice",
        "registerDownloadJob",
        "GETFileJob*",
        "unregisterDownloadJob",
        "absoluteLimitTimerExpired",
        "relativeUploadMeasuringTimerExpired",
        "relativeUploadDelayTimerExpired",
        "relativeDownloadMeasuringTimerExpired",
        "relativeDownloadDelayTimerExpired"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'registerUploadDevice'
        QtMocHelpers::SlotData<void(UploadDevice *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'unregisterUploadDevice'
        QtMocHelpers::SlotData<void(QObject *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 2 },
        }}),
        // Slot 'registerDownloadJob'
        QtMocHelpers::SlotData<void(GETFileJob *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'unregisterDownloadJob'
        QtMocHelpers::SlotData<void(GETFileJob *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 2 },
        }}),
        // Slot 'absoluteLimitTimerExpired'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'relativeUploadMeasuringTimerExpired'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'relativeUploadDelayTimerExpired'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'relativeDownloadMeasuringTimerExpired'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'relativeDownloadDelayTimerExpired'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BandwidthManager, qt_meta_tag_ZN3OCC16BandwidthManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::BandwidthManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC16BandwidthManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC16BandwidthManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC16BandwidthManagerE_t>.metaTypes,
    nullptr
} };

void OCC::BandwidthManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BandwidthManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->registerUploadDevice((*reinterpret_cast<std::add_pointer_t<UploadDevice*>>(_a[1]))); break;
        case 1: _t->unregisterUploadDevice((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 2: _t->registerDownloadJob((*reinterpret_cast<std::add_pointer_t<GETFileJob*>>(_a[1]))); break;
        case 3: _t->unregisterDownloadJob((*reinterpret_cast<std::add_pointer_t<GETFileJob*>>(_a[1]))); break;
        case 4: _t->absoluteLimitTimerExpired(); break;
        case 5: _t->relativeUploadMeasuringTimerExpired(); break;
        case 6: _t->relativeUploadDelayTimerExpired(); break;
        case 7: _t->relativeDownloadMeasuringTimerExpired(); break;
        case 8: _t->relativeDownloadDelayTimerExpired(); break;
        default: ;
        }
    }
}

const QMetaObject *OCC::BandwidthManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::BandwidthManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC16BandwidthManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::BandwidthManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
