/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<fonticon.h>)
#  include <fonticon.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
namespace OCC {
Q_QMLTYPE_EXPORT void qml_register_types_eu_OpenCloud_resources()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<OCC::Resources::FontIcon>("eu.OpenCloud.resources", 1);
    qmlRegisterEnum<OCC::Resources::FontIcon::DefaultGlyphes>("OCC::Resources::FontIcon::DefaultGlyphes");
    qmlRegisterEnum<OCC::Resources::FontIcon::FontFamily>("OCC::Resources::FontIcon::FontFamily");
    qmlRegisterEnum<OCC::Resources::FontIcon::Size>("OCC::Resources::FontIcon::Size");
    QMetaType::fromType<OCC::Resources::FontIconDerived>().id();
    qmlRegisterNamespaceAndRevisions(&OCC::Resources::FontIconDerived::staticMetaObject, "eu.OpenCloud.resources", 1, nullptr, &OCC::Resources::FontIconDerivedForeign::staticMetaObject, nullptr);
    QMetaType::fromType<QIcon>().id();
    qmlRegisterEnum<QIcon::Mode>("QIcon::Mode");
    qmlRegisterEnum<QIcon::State>("QIcon::State");
    qmlRegisterEnum<QIcon::ThemeIcon>("QIcon::ThemeIcon");
    QT_WARNING_POP
    qmlRegisterModule("eu.OpenCloud.resources", 1, 0);
}

static const QQmlModuleRegistration euOpenCloudresourcesRegistration("eu.OpenCloud.resources", qml_register_types_eu_OpenCloud_resources);
} // namespace OCC
