/********************************************************************************
** Form generated from reading UI file 'setupwizardwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETUPWIZARDWIDGET_H
#define UI_SETUPWIZARDWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "3rdparty/QProgressIndicator/QProgressIndicator.h"
#include "gui/newwizard/navigation.h"

QT_BEGIN_NAMESPACE

class Ui_SetupWizardWidget
{
public:
    QVBoxLayout *verticalLayout;
    QStackedWidget *contentWidget;
    QWidget *transitionPage;
    QGridLayout *gridLayout;
    QProgressIndicator *transitionProgressIndicator;
    QVBoxLayout *verticalLayout_2;
    QWidget *errorMessageWidget;
    QHBoxLayout *horizontalLayout;
    QLabel *warningPixmap;
    QLabel *errorMessageLabel;
    QPushButton *errorMessageDismissButton;
    QHBoxLayout *bottomButtonsLayout;
    QPushButton *cancelButton;
    QSpacerItem *horizontalSpacer;
    OCC::Wizard::Navigation *navigation;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *backButton;
    QPushButton *nextButton;

    void setupUi(QWidget *SetupWizardWidget)
    {
        if (SetupWizardWidget->objectName().isEmpty())
            SetupWizardWidget->setObjectName("SetupWizardWidget");
        SetupWizardWidget->resize(750, 500);
        verticalLayout = new QVBoxLayout(SetupWizardWidget);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        contentWidget = new QStackedWidget(SetupWizardWidget);
        contentWidget->setObjectName("contentWidget");
        transitionPage = new QWidget();
        transitionPage->setObjectName("transitionPage");
        gridLayout = new QGridLayout(transitionPage);
        gridLayout->setObjectName("gridLayout");
        transitionProgressIndicator = new QProgressIndicator(transitionPage);
        transitionProgressIndicator->setObjectName("transitionProgressIndicator");

        gridLayout->addWidget(transitionProgressIndicator, 0, 0, 1, 1);

        contentWidget->addWidget(transitionPage);

        verticalLayout->addWidget(contentWidget);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(0);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(9, -1, 9, 9);
        errorMessageWidget = new QWidget(SetupWizardWidget);
        errorMessageWidget->setObjectName("errorMessageWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(errorMessageWidget->sizePolicy().hasHeightForWidth());
        errorMessageWidget->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(errorMessageWidget);
        horizontalLayout->setObjectName("horizontalLayout");
        warningPixmap = new QLabel(errorMessageWidget);
        warningPixmap->setObjectName("warningPixmap");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(warningPixmap->sizePolicy().hasHeightForWidth());
        warningPixmap->setSizePolicy(sizePolicy1);
        warningPixmap->setMinimumSize(QSize(64, 64));
        warningPixmap->setMaximumSize(QSize(64, 64));
        warningPixmap->setPixmap(QPixmap(QString::fromUtf8(":/client/resources/light/warning.svg")));
        warningPixmap->setScaledContents(true);

        horizontalLayout->addWidget(warningPixmap);

        errorMessageLabel = new QLabel(errorMessageWidget);
        errorMessageLabel->setObjectName("errorMessageLabel");
        errorMessageLabel->setTextFormat(Qt::TextFormat::RichText);

        horizontalLayout->addWidget(errorMessageLabel);

        errorMessageDismissButton = new QPushButton(errorMessageWidget);
        errorMessageDismissButton->setObjectName("errorMessageDismissButton");

        horizontalLayout->addWidget(errorMessageDismissButton);

        horizontalLayout->setStretch(1, 1);

        verticalLayout_2->addWidget(errorMessageWidget);

        bottomButtonsLayout = new QHBoxLayout();
        bottomButtonsLayout->setObjectName("bottomButtonsLayout");
        bottomButtonsLayout->setContentsMargins(-1, 9, -1, -1);
        cancelButton = new QPushButton(SetupWizardWidget);
        cancelButton->setObjectName("cancelButton");

        bottomButtonsLayout->addWidget(cancelButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        bottomButtonsLayout->addItem(horizontalSpacer);

        navigation = new OCC::Wizard::Navigation(SetupWizardWidget);
        navigation->setObjectName("navigation");

        bottomButtonsLayout->addWidget(navigation);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        bottomButtonsLayout->addItem(horizontalSpacer_2);

        backButton = new QPushButton(SetupWizardWidget);
        backButton->setObjectName("backButton");

        bottomButtonsLayout->addWidget(backButton);

        nextButton = new QPushButton(SetupWizardWidget);
        nextButton->setObjectName("nextButton");
        nextButton->setText(QString::fromUtf8("placeholder"));

        bottomButtonsLayout->addWidget(nextButton);


        verticalLayout_2->addLayout(bottomButtonsLayout);


        verticalLayout->addLayout(verticalLayout_2);


        retranslateUi(SetupWizardWidget);

        QMetaObject::connectSlotsByName(SetupWizardWidget);
    } // setupUi

    void retranslateUi(QWidget *SetupWizardWidget)
    {
        SetupWizardWidget->setWindowTitle(QCoreApplication::translate("SetupWizardWidget", "Add New Account", nullptr));
        errorMessageDismissButton->setText(QCoreApplication::translate("SetupWizardWidget", "&Dismiss", nullptr));
        cancelButton->setText(QCoreApplication::translate("SetupWizardWidget", "&Cancel", nullptr));
        backButton->setText(QCoreApplication::translate("SetupWizardWidget", "< Back", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SetupWizardWidget: public Ui_SetupWizardWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETUPWIZARDWIDGET_H
