/****************************************************************************
** Meta object code from reading C++ file 'folder.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../opencloud-desktop/src/gui/folder.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'folder.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC6FolderE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Folder::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC6FolderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Folder",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Folders can only be created by the FolderManager",
        "syncStateChange",
        "",
        "syncFinished",
        "SyncResult",
        "result",
        "syncPausedChanged",
        "Folder*",
        "paused",
        "canSyncChanged",
        "spaceChanged",
        "isReadyChanged",
        "isSyncRunningChanged",
        "watchedFileChangedExternally",
        "path",
        "openInWebBrowser",
        "startSync",
        "slotDiscardDownloadProgress",
        "slotWipeErrorBlacklist",
        "slotWatchedPathsChanged",
        "QSet<QString>",
        "paths",
        "ChangeReason",
        "reason",
        "slotNextSyncFullLocalDiscovery",
        "schedulePathForLocalDiscovery",
        "relativePath",
        "reloadExcludes",
        "slotSyncFinished",
        "slotSyncError",
        "message",
        "ErrorCategory",
        "category",
        "slotItemCompleted",
        "SyncFileItemPtr",
        "slotFolderConflicts",
        "folder",
        "conflictPaths",
        "warnOnNewExcludedItem",
        "SyncJournalFileRecord",
        "record",
        "QStringView",
        "slotWatcherUnreliable",
        "space",
        "GraphApi::Space*",
        "webDavUrl",
        "QUrl",
        "isReady",
        "isSyncPaused",
        "isSyncRunning",
        "isDeployed",
        "vfsMode",
        "Vfs::Mode",
        "Other",
        "UnLock"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'syncStateChange'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'syncFinished'
        QtMocHelpers::SignalData<void(const SyncResult &)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'syncPausedChanged'
        QtMocHelpers::SignalData<void(Folder *, bool)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 8 }, { QMetaType::Bool, 14 },
        }}),
        // Signal 'canSyncChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'spaceChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isReadyChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isSyncRunningChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'watchedFileChangedExternally'
        QtMocHelpers::SignalData<void(const QString &)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 },
        }}),
        // Slot 'openInWebBrowser'
        QtMocHelpers::SlotData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startSync'
        QtMocHelpers::SlotData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDiscardDownloadProgress'
        QtMocHelpers::SlotData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotWipeErrorBlacklist'
        QtMocHelpers::SlotData<int()>(24, 8, QMC::AccessPublic, QMetaType::Int),
        // Slot 'slotWatchedPathsChanged'
        QtMocHelpers::SlotData<void(const QSet<QString> &, enum ChangeReason)>(25, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 27 }, { 0x80000000 | 28, 29 },
        }}),
        // Slot 'slotNextSyncFullLocalDiscovery'
        QtMocHelpers::SlotData<void()>(30, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'schedulePathForLocalDiscovery'
        QtMocHelpers::SlotData<void(const QString &)>(31, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 },
        }}),
        // Slot 'reloadExcludes'
        QtMocHelpers::SlotData<bool()>(33, 8, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotSyncFinished'
        QtMocHelpers::SlotData<void(bool)>(34, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'slotSyncError'
        QtMocHelpers::SlotData<void(const QString &, ErrorCategory)>(35, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 36 }, { 0x80000000 | 37, 38 },
        }}),
        // Slot 'slotSyncError'
        QtMocHelpers::SlotData<void(const QString &)>(35, 8, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 36 },
        }}),
        // Slot 'slotItemCompleted'
        QtMocHelpers::SlotData<void(const SyncFileItemPtr &)>(39, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 40, 8 },
        }}),
        // Slot 'slotFolderConflicts'
        QtMocHelpers::SlotData<void(Folder *, const QStringList &)>(41, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 42 }, { QMetaType::QStringList, 43 },
        }}),
        // Slot 'warnOnNewExcludedItem'
        QtMocHelpers::SlotData<void(const SyncJournalFileRecord &, QStringView)>(44, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 45, 46 }, { 0x80000000 | 47, 20 },
        }}),
        // Slot 'slotWatcherUnreliable'
        QtMocHelpers::SlotData<void(const QString &)>(48, 8, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 36 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'space'
        QtMocHelpers::PropertyData<GraphApi::Space*>(49, 0x80000000 | 50, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'path'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'webDavUrl'
        QtMocHelpers::PropertyData<QUrl>(51, 0x80000000 | 52, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'isReady'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
        // property 'isSyncPaused'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'isSyncRunning'
        QtMocHelpers::PropertyData<bool>(55, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'isDeployed'
        QtMocHelpers::PropertyData<bool>(56, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'vfsMode'
        QtMocHelpers::PropertyData<Vfs::Mode>(57, 0x80000000 | 58, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ChangeReason'
        QtMocHelpers::EnumData<enum ChangeReason>(28, 28, QMC::EnumIsScoped).add({
            {   59, ChangeReason::Other },
            {   60, ChangeReason::UnLock },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Folder, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN3OCC6FolderE[] = {
    QMetaObject::SuperData::link<OCC::Vfs::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject OCC::Folder::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6FolderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6FolderE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN3OCC6FolderE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC6FolderE_t>.metaTypes,
    nullptr
} };

void OCC::Folder::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Folder *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->syncStateChange(); break;
        case 1: _t->syncFinished((*reinterpret_cast<std::add_pointer_t<SyncResult>>(_a[1]))); break;
        case 2: _t->syncPausedChanged((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 3: _t->canSyncChanged(); break;
        case 4: _t->spaceChanged(); break;
        case 5: _t->isReadyChanged(); break;
        case 6: _t->isSyncRunningChanged(); break;
        case 7: _t->watchedFileChangedExternally((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->openInWebBrowser(); break;
        case 9: _t->startSync(); break;
        case 10: _t->slotDiscardDownloadProgress(); break;
        case 11: { int _r = _t->slotWipeErrorBlacklist();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->slotWatchedPathsChanged((*reinterpret_cast<std::add_pointer_t<QSet<QString>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum ChangeReason>>(_a[2]))); break;
        case 13: _t->slotNextSyncFullLocalDiscovery(); break;
        case 14: _t->schedulePathForLocalDiscovery((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: { bool _r = _t->reloadExcludes();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->slotSyncFinished((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->slotSyncError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ErrorCategory>>(_a[2]))); break;
        case 18: _t->slotSyncError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 19: _t->slotItemCompleted((*reinterpret_cast<std::add_pointer_t<SyncFileItemPtr>>(_a[1]))); break;
        case 20: _t->slotFolderConflicts((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 21: _t->warnOnNewExcludedItem((*reinterpret_cast<std::add_pointer_t<SyncJournalFileRecord>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringView>>(_a[2]))); break;
        case 22: _t->slotWatcherUnreliable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QString> >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Folder* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::syncStateChange, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(const SyncResult & )>(_a, &Folder::syncFinished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(Folder * , bool )>(_a, &Folder::syncPausedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::canSyncChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::spaceChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::isReadyChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)()>(_a, &Folder::isSyncRunningChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Folder::*)(const QString & )>(_a, &Folder::watchedFileChangedExternally, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<GraphApi::Space**>(_v) = _t->space(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->path(); break;
        case 2: *reinterpret_cast<QUrl*>(_v) = _t->webDavUrl(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isReady(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isSyncPaused(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->isSyncRunning(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isDeployed(); break;
        case 7: *reinterpret_cast<Vfs::Mode*>(_v) = _t->vfsMode(); break;
        default: break;
        }
    }
}

const QMetaObject *OCC::Folder::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::Folder::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC6FolderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OCC::Folder::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void OCC::Folder::syncStateChange()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OCC::Folder::syncFinished(const SyncResult & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OCC::Folder::syncPausedChanged(Folder * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OCC::Folder::canSyncChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OCC::Folder::spaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OCC::Folder::isReadyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OCC::Folder::isSyncRunningChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OCC::Folder::watchedFileChangedExternally(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
