/****************************************************************************
** Meta object code from reading C++ file 'csync.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../opencloud-desktop/src/libsync/csync.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'csync.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10CSyncEnumsE_t {};
} // unnamed namespace

template <> constexpr inline auto CSyncEnums::qt_create_metaobjectdata<qt_meta_tag_ZN10CSyncEnumsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CSyncEnums",
        "SyncInstruction",
        "CSYNC_INSTRUCTION_NONE",
        "CSYNC_INSTRUCTION_REMOVE",
        "CSYNC_INSTRUCTION_RENAME",
        "CSYNC_INSTRUCTION_NEW",
        "CSYNC_INSTRUCTION_CONFLICT",
        "CSYNC_INSTRUCTION_IGNORE",
        "CSYNC_INSTRUCTION_SYNC",
        "CSYNC_INSTRUCTION_ERROR",
        "CSYNC_INSTRUCTION_TYPE_CHANGE",
        "CSYNC_INSTRUCTION_UPDATE_METADATA",
        "ItemType",
        "ItemTypeFile",
        "ItemTypeSymLink",
        "ItemTypeDirectory",
        "ItemTypeUnsupported",
        "ItemTypeVirtualFile",
        "ItemTypeVirtualFileDownload",
        "ItemTypeVirtualFileDehydration"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SyncInstruction'
        QtMocHelpers::EnumData<SyncInstruction>(1, 1, QMC::EnumFlags{}).add({
            {    2, SyncInstruction::CSYNC_INSTRUCTION_NONE },
            {    3, SyncInstruction::CSYNC_INSTRUCTION_REMOVE },
            {    4, SyncInstruction::CSYNC_INSTRUCTION_RENAME },
            {    5, SyncInstruction::CSYNC_INSTRUCTION_NEW },
            {    6, SyncInstruction::CSYNC_INSTRUCTION_CONFLICT },
            {    7, SyncInstruction::CSYNC_INSTRUCTION_IGNORE },
            {    8, SyncInstruction::CSYNC_INSTRUCTION_SYNC },
            {    9, SyncInstruction::CSYNC_INSTRUCTION_ERROR },
            {   10, SyncInstruction::CSYNC_INSTRUCTION_TYPE_CHANGE },
            {   11, SyncInstruction::CSYNC_INSTRUCTION_UPDATE_METADATA },
        }),
        // enum 'ItemType'
        QtMocHelpers::EnumData<ItemType>(12, 12, QMC::EnumFlags{}).add({
            {   13, ItemType::ItemTypeFile },
            {   14, ItemType::ItemTypeSymLink },
            {   15, ItemType::ItemTypeDirectory },
            {   16, ItemType::ItemTypeUnsupported },
            {   17, ItemType::ItemTypeVirtualFile },
            {   18, ItemType::ItemTypeVirtualFileDownload },
            {   19, ItemType::ItemTypeVirtualFileDehydration },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN10CSyncEnumsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN10CSyncEnumsE =
    CSyncEnums::qt_create_metaobjectdata<qt_meta_tag_ZN10CSyncEnumsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN10CSyncEnumsE =
    qt_staticMetaObjectContent_ZN10CSyncEnumsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN10CSyncEnumsE =
    qt_staticMetaObjectContent_ZN10CSyncEnumsE.relocatingData;

Q_CONSTINIT const QMetaObject CSyncEnums::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN10CSyncEnumsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN10CSyncEnumsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN10CSyncEnumsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
