/****************************************************************************
** Meta object code from reading C++ file 'fonticon.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../opencloud-desktop/src/resources/fonticon.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fonticon.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC9Resources8FontIconE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Resources::FontIcon::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9Resources8FontIconE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Resources::FontIcon",
        "QML.Element",
        "fontIcon",
        "DefaultGlyphes",
        "Question",
        "Warning",
        "Info",
        "FontFamily",
        "FontAwesome",
        "RemixIcon",
        "Size",
        "Normal",
        "Half"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DefaultGlyphes'
        QtMocHelpers::EnumData<enum DefaultGlyphes>(3, 3, QMC::EnumIsScoped).add({
            {    4, DefaultGlyphes::Question },
            {    5, DefaultGlyphes::Warning },
            {    6, DefaultGlyphes::Info },
        }),
        // enum 'FontFamily'
        QtMocHelpers::EnumData<enum FontFamily>(7, 7, QMC::EnumIsScoped).add({
            {    8, FontFamily::FontAwesome },
            {    9, FontFamily::RemixIcon },
        }),
        // enum 'Size'
        QtMocHelpers::EnumData<enum Size>(10, 10, QMC::EnumIsScoped).add({
            {   11, Size::Normal },
            {   12, Size::Half },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<FontIcon, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OCC::Resources::FontIcon::staticMetaObject = { {
    QtPrivate::MetaObjectForType<QIcon>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9Resources8FontIconE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9Resources8FontIconE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC9Resources8FontIconE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN3OCC9Resources15FontIconDerivedE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Resources::FontIconDerived::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9Resources15FontIconDerivedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Resources::FontIconDerived"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FontIconDerived, qt_meta_tag_ZN3OCC9Resources15FontIconDerivedE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OCC::Resources::FontIconDerived::staticMetaObject = { {
    QtPrivate::MetaObjectForType<FontIcon>::value,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9Resources15FontIconDerivedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC9Resources15FontIconDerivedE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC9Resources15FontIconDerivedE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN3OCC9Resources22FontIconDerivedForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::Resources::FontIconDerivedForeign::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9Resources22FontIconDerivedForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::Resources::FontIconDerivedForeign",
        "QML.Element",
        "FontIcon",
        "QML.Foreign",
        "Resources::FontIconDerived",
        "QML.ForeignIsNamespace",
        "true"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN3OCC9Resources22FontIconDerivedForeignE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN3OCC9Resources22FontIconDerivedForeignE =
    OCC::Resources::FontIconDerivedForeign::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC9Resources22FontIconDerivedForeignE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN3OCC9Resources22FontIconDerivedForeignE =
    qt_staticMetaObjectContent_ZN3OCC9Resources22FontIconDerivedForeignE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN3OCC9Resources22FontIconDerivedForeignE =
    qt_staticMetaObjectContent_ZN3OCC9Resources22FontIconDerivedForeignE.relocatingData;

Q_CONSTINIT const QMetaObject OCC::Resources::FontIconDerivedForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN3OCC9Resources22FontIconDerivedForeignE.stringdata,
    qt_staticMetaObjectStaticContent_ZN3OCC9Resources22FontIconDerivedForeignE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN3OCC9Resources22FontIconDerivedForeignE.metaTypes,
    nullptr
} };

QT_WARNING_POP
