/********************************************************************************
** Form generated from reading UI file 'serverurlsetupwizardpage.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SERVERURLSETUPWIZARDPAGE_H
#define UI_SERVERURLSETUPWIZARDPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ServerUrlSetupWizardPage
{
public:
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer_2;
    QLabel *logoLabel;
    QSpacerItem *verticalSpacer_4;
    QLabel *welcomeTextLabel;
    QSpacerItem *verticalSpacer_3;
    QLabel *serverUrlLabel;
    QSpacerItem *verticalSpacer_5;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QLineEdit *urlLineEdit;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *ServerUrlSetupWizardPage)
    {
        if (ServerUrlSetupWizardPage->objectName().isEmpty())
            ServerUrlSetupWizardPage->setObjectName("ServerUrlSetupWizardPage");
        ServerUrlSetupWizardPage->resize(492, 364);
        verticalLayout = new QVBoxLayout(ServerUrlSetupWizardPage);
        verticalLayout->setObjectName("verticalLayout");
        verticalSpacer_2 = new QSpacerItem(20, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        verticalLayout->addItem(verticalSpacer_2);

        logoLabel = new QLabel(ServerUrlSetupWizardPage);
        logoLabel->setObjectName("logoLabel");
        logoLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(logoLabel);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        verticalLayout->addItem(verticalSpacer_4);

        welcomeTextLabel = new QLabel(ServerUrlSetupWizardPage);
        welcomeTextLabel->setObjectName("welcomeTextLabel");
        welcomeTextLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(welcomeTextLabel);

        verticalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        verticalLayout->addItem(verticalSpacer_3);

        serverUrlLabel = new QLabel(ServerUrlSetupWizardPage);
        serverUrlLabel->setObjectName("serverUrlLabel");
        serverUrlLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(serverUrlLabel);

        verticalSpacer_5 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        verticalLayout->addItem(verticalSpacer_5);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(10, 20, QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        urlLineEdit = new QLineEdit(ServerUrlSetupWizardPage);
        urlLineEdit->setObjectName("urlLineEdit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(urlLineEdit->sizePolicy().hasHeightForWidth());
        urlLineEdit->setSizePolicy(sizePolicy);
        urlLineEdit->setMinimumSize(QSize(400, 0));
        urlLineEdit->setAlignment(Qt::AlignmentFlag::AlignCenter);

        horizontalLayout->addWidget(urlLineEdit);

        horizontalSpacer_2 = new QSpacerItem(10, 20, QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        horizontalLayout->setStretch(0, 1);
        horizontalLayout->setStretch(1, 2);
        horizontalLayout->setStretch(2, 1);

        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        serverUrlLabel->setBuddy(urlLineEdit);
#endif // QT_CONFIG(shortcut)

        retranslateUi(ServerUrlSetupWizardPage);

        QMetaObject::connectSlotsByName(ServerUrlSetupWizardPage);
    } // setupUi

    void retranslateUi(QWidget *ServerUrlSetupWizardPage)
    {
        ServerUrlSetupWizardPage->setWindowTitle(QCoreApplication::translate("ServerUrlSetupWizardPage", "Form", nullptr));
        welcomeTextLabel->setText(QCoreApplication::translate("ServerUrlSetupWizardPage", "Welcome", nullptr));
        serverUrlLabel->setText(QCoreApplication::translate("ServerUrlSetupWizardPage", "What is your server's address?", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ServerUrlSetupWizardPage: public Ui_ServerUrlSetupWizardPage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SERVERURLSETUPWIZARDPAGE_H
