// This file is autogenerated by CMake. Do not edit.

#include <QtCore/qtsymbolmacros.h>
#include <QtQml/qqmlextensionplugin.h>



namespace OCC {

QT_DECLARE_EXTERN_SYMBOL_VOID(qml_register_types_eu_OpenCloud_libsync)

class eu_OpenCloud_libsyncPlugin : public QQmlEngineExtensionPlugin
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID QQmlEngineExtensionInterface_iid)

public:
    eu_OpenCloud_libsyncPlugin(QObject *parent = nullptr) : QQmlEngineExtensionPlugin(parent)
    {

QT_KEEP_SYMBOL(qml_register_types_eu_OpenCloud_libsync)
    }
};

} // namespace OCC

#include "libsyncplugin_eu_OpenCloud_libsyncPlugin.moc"
