/********************************************************************************
** Form generated from reading UI file 'accountsettings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACCOUNTSETTINGS_H
#define UI_ACCOUNTSETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "QProgressIndicator.h"
#include "gui/qmlutils.h"

namespace OCC {

class Ui_AccountSettings
{
public:
    QVBoxLayout *verticalLayout_2;
    QStackedWidget *stackedWidget;
    QWidget *loadingPage;
    QVBoxLayout *verticalLayout_4;
    QSpacerItem *verticalSpacer_2;
    QLabel *label;
    QHBoxLayout *horizontalLayout_6;
    QProgressIndicator *spinner;
    QSpacerItem *verticalSpacer_3;
    OCC::QmlUtils::OCQuickWidget *quickWidget;

    void setupUi(QWidget *OCC__AccountSettings)
    {
        if (OCC__AccountSettings->objectName().isEmpty())
            OCC__AccountSettings->setObjectName("OCC__AccountSettings");
        OCC__AccountSettings->resize(678, 557);
        verticalLayout_2 = new QVBoxLayout(OCC__AccountSettings);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        stackedWidget = new QStackedWidget(OCC__AccountSettings);
        stackedWidget->setObjectName("stackedWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(5);
        sizePolicy.setHeightForWidth(stackedWidget->sizePolicy().hasHeightForWidth());
        stackedWidget->setSizePolicy(sizePolicy);
        loadingPage = new QWidget();
        loadingPage->setObjectName("loadingPage");
        verticalLayout_4 = new QVBoxLayout(loadingPage);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_2);

        label = new QLabel(loadingPage);
        label->setObjectName("label");

        verticalLayout_4->addWidget(label);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        spinner = new QProgressIndicator(loadingPage);
        spinner->setObjectName("spinner");

        horizontalLayout_6->addWidget(spinner);


        verticalLayout_4->addLayout(horizontalLayout_6);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_3);

        stackedWidget->addWidget(loadingPage);
        quickWidget = new OCC::QmlUtils::OCQuickWidget();
        quickWidget->setObjectName("quickWidget");
        stackedWidget->addWidget(quickWidget);

        verticalLayout_2->addWidget(stackedWidget);


        retranslateUi(OCC__AccountSettings);

        stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(OCC__AccountSettings);
    } // setupUi

    void retranslateUi(QWidget *OCC__AccountSettings)
    {
        OCC__AccountSettings->setWindowTitle(QCoreApplication::translate("OCC::AccountSettings", "Form", nullptr));
        label->setText(QCoreApplication::translate("OCC::AccountSettings", "Preparing the account", nullptr));
#if QT_CONFIG(accessibility)
        quickWidget->setAccessibleName(QCoreApplication::translate("OCC::AccountSettings", "Sync connections", nullptr));
#endif // QT_CONFIG(accessibility)
    } // retranslateUi

};

} // namespace OCC

namespace OCC {
namespace Ui {
    class AccountSettings: public Ui_AccountSettings {};
} // namespace Ui
} // namespace OCC

#endif // UI_ACCOUNTSETTINGS_H
