/****************************************************************************
** Meta object code from reading C++ file 'folderstatusmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../opencloud-desktop/src/gui/folderstatusmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'folderstatusmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3OCC17FolderStatusModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OCC::FolderStatusModel::qt_create_metaobjectdata<qt_meta_tag_ZN3OCC17FolderStatusModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OCC::FolderStatusModel",
        "QML.Element",
        "auto",
        "slotUpdateFolderState",
        "",
        "Folder*",
        "resetFolders",
        "slotSetProgress",
        "ProgressInfo",
        "progress",
        "f",
        "slotFolderSyncStateChange",
        "Roles",
        "AccessibleDescriptionRole",
        "DisplayName",
        "Subtitle",
        "FolderErrorMsg",
        "SyncProgressOverallPercent",
        "SyncProgressOverallString",
        "SyncProgressItemString",
        "Priority",
        "Quota",
        "FolderStatusIcon",
        "Folder"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotUpdateFolderState'
        QtMocHelpers::SlotData<void(Folder *)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 4 },
        }}),
        // Slot 'resetFolders'
        QtMocHelpers::SlotData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSetProgress'
        QtMocHelpers::SlotData<void(const ProgressInfo &, Folder *)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 5, 10 },
        }}),
        // Slot 'slotFolderSyncStateChange'
        QtMocHelpers::SlotData<void(Folder *)>(11, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Roles'
        QtMocHelpers::EnumData<enum Roles>(12, 12, QMC::EnumIsScoped).add({
            {   13, Roles::AccessibleDescriptionRole },
            {   14, Roles::DisplayName },
            {   15, Roles::Subtitle },
            {   16, Roles::FolderErrorMsg },
            {   17, Roles::SyncProgressOverallPercent },
            {   18, Roles::SyncProgressOverallString },
            {   19, Roles::SyncProgressItemString },
            {   20, Roles::Priority },
            {   21, Roles::Quota },
            {   22, Roles::FolderStatusIcon },
            {   23, Roles::Folder },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<FolderStatusModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OCC::FolderStatusModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17FolderStatusModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17FolderStatusModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3OCC17FolderStatusModelE_t>.metaTypes,
    nullptr
} };

void OCC::FolderStatusModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FolderStatusModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotUpdateFolderState((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        case 1: _t->resetFolders(); break;
        case 2: _t->slotSetProgress((*reinterpret_cast<std::add_pointer_t<ProgressInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[2]))); break;
        case 3: _t->slotFolderSyncStateChange((*reinterpret_cast<std::add_pointer_t<Folder*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *OCC::FolderStatusModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OCC::FolderStatusModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3OCC17FolderStatusModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int OCC::FolderStatusModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
QT_WARNING_POP
